<?php
/**
 * All-in-one Performance Accelerator plugin file.
 *
 * Copyright (C) 2010-2020, Smackcoders Inc - info@smackcoders.com
 */

namespace Smackcoders\AIOACC;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class SmackQuery {

	private $display = true;
	private $starttime, $servertime, $css, $ajax;
	protected static $instance = null,$plugin;

	public static function getInstance() {
      
		if ( null == self::$instance ) {
			self::$instance = new self;
			self::$plugin = Plugin::getInstance();
			self::$instance->doHooks();
		}
		return self::$instance;
	}

	public function doHooks(){
		
		
	}

	function __construct() {
		$css='.query-dd{display:block;line-height:24px;background-color:#c4c4c4;color:#000;font-family:Helvetica,Arial,sans-serif;font-size:18px;text-align:center}.query-bs{margin:0 45px}.query-ss{margin:0 4px;border-left:5px solid #999}#query_dd{position:fixed;left:50%;bottom:0;z-index:9999998;margin-left:-160px;width:475px;height:26px;white-space:nowrap;overflow:hidden;background-color:rgb(159 188 253 / 50%);border-radius:6px;margin-bottom:21px;padding:2px}';
		$this->css = '<style type="text/css" scoped>' . str_replace(array("\r","\n","\t"), '', $css);
		$this->ajax = 'false';

		// give plugins/themes lots of time to set NO_SmackQuery_DISPLAY constant
		add_action('init', array($this, 'hook_setup'), 9999);

		// hooks for things where usage display should be suppressed
		add_filter('wp_xmlrpc_server_class', array($this, 'no_query_display'));
		add_filter('rest_send_nocache_headers', array($this, 'no_query_display'));
	}

	function hook_setup() {
		// allow a theme/plugin to disable SmackQuery display
		if (!defined('NO_SmackQuery_DISPLAY')) {
			if (defined('WP_ADMIN')) {
				add_action('admin_init', array($this, 'smack_time_to_first_byte'), 9999);
				add_action('admin_footer', function(){register_shutdown_function(array($this, 'query_usage'));}, 9999);
			}
			else {
				add_action('wp_loaded', array($this, 'smack_time_to_first_byte'), 9999);
				add_action('wp_footer', function(){register_shutdown_function(array($this, 'query_usage'));}, 9999);
			}
		}
	}

	// function works for filter or action hooks
	function no_query_display($val) {
		$this->display = false;
		return $val;
	}

	function smack_time_to_first_byte() {
		// theme display/customizer does some, um, unusual things...
		if (is_admin()) {
			$this->css .= '.wrap .theme-overlay .theme-wrap,.wrap .theme-overlay .theme-backdrop,.wrap .wp-full-overlay-sidebar-content,#customize-preview iframe{bottom:24px}';
		}
		$server_array = filter_input_array(INPUT_SERVER, FILTER_SANITIZE_STRING);
		// PHP 5.4+ provides this value
		if (!empty($server_array['REQUEST_TIME_FLOAT'])) {
			$this->starttime = $server_array['REQUEST_TIME_FLOAT'];
			$this->servertime = strval(round(microtime(true) - $this->starttime, 2)) . '<span class="query-ss"></span>';
		}
	}

	function query_usage() {
		if ($this->display && !defined('WP_INSTALLING') && (!defined('DOING_AJAX') || (defined('DOING_AJAX') && $this->ajax)) && current_user_can('update_core')) {
			global $wpdb;

			$precision = 0;
			$memory_usage = memory_get_peak_usage() / 1048576;
			if ($memory_usage < 10) {
				$precision = 2;
			}
			else if ($memory_usage < 100) {
				$precision = 1;
			}

			$memory_usage = round($memory_usage, $precision);
			$time_usage = (empty($this->starttime)) ? '' : $this->servertime . round(microtime(true) - $this->starttime, 2);
           
            echo ((defined('DOING_AJAX')) ? '' : $this->css . '</style><div id="query_dd_spacer"></div>') . '<div class="query-dd"' . ((defined('DOING_AJAX') && $this->ajax) ? '>' : ' id="query_dd">') . "Memory:{$memory_usage}M<span class=\"query-bs\">Time:{$time_usage}s</span>Queries:{$wpdb->num_queries}Q</div>";
		}
	}
}

add_action('plugins_loaded', function(){new SmackQuery();}, 1);
