<?php
/**
 * All-in-one Performance Accelerator plugin file.
 *
 * Copyright (C) 2010-2020, Smackcoders Inc - info@smackcoders.com
 */

namespace Smackcoders\AIOACC;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}
require_once(__DIR__.'/scheduleDBCleanup.php');
	
class OptimizeDB
{
	protected static $instance = null,$plugin;
	
	public function __construct()
	{
		   
	}

	
	public static function getInstance() {
      
		if ( null == self::$instance ) {
			self::$instance = new self;
			self::$plugin = Plugin::getInstance();
			self::$instance->doHooks();
		}
		return self::$instance;
	}

	
	public function doHooks(){
        add_action('wp_ajax_save_database_optimization_options', array($this,'get_db_options'));
        add_action('wp_ajax_get_database_optimization_options', array($this,'get_count'));   
    }

	public function get_count(){
        global $wpdb;
        $get_revisions   = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM {$wpdb->prefix}posts WHERE post_type=%s", 'revision' ) );
        $get_trash   = $wpdb->get_var($wpdb->prepare ("SELECT COUNT(*) FROM {$wpdb->prefix}posts WHERE post_status=%s", 'trash'));
        $get_draft   = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM {$wpdb->prefix}posts WHERE post_status=%s", 'draft'));
        $get_spam_comments  = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM {$wpdb->prefix}comments WHERE comment_approved=%s", 'spam') );
        $get_trash_comments  = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM {$wpdb->prefix}comments WHERE comment_approved=%s", 'trash'));
        $server_array = filter_input_array(INPUT_SERVER, FILTER_SANITIZE_STRING);
        $time  = isset( $server_array['REQUEST_TIME'] ) ? (int) $server_array['REQUEST_TIME'] : time();
        $expired_transient = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(option_name) FROM $wpdb->options WHERE option_name LIKE %s AND option_value < %d", $wpdb->esc_like( '_transient_timeout' ) . '%', $time ) );
        $all_transient = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(option_id) FROM $wpdb->options WHERE option_name LIKE %s OR option_name LIKE %s", $wpdb->esc_like( '_transient_' ) . '%', $wpdb->esc_like( '_site_transient_' ) . '%' ) );
        $optimize_database = $wpdb->get_var("SELECT COUNT(table_name) FROM information_schema.tables WHERE table_schema = '" . DB_NAME . "' and Engine <> 'InnoDB' and data_free > 0");
        $result['revision']=(int)$get_revisions;
        $result['trash']=(int)$get_trash;
        $result['draft']=(int)$get_draft;
        $result['spam']=(int)$get_spam_comments;
        $result['trash_comments']=(int)$get_trash_comments;
        $result['expired_transient']=(int)$expired_transient;
        $result['all_transient']=(int)$all_transient;
        $result['optimize_database']=(int)$optimize_database;
        $result['revision_status']=get_option('clean_revisions')=== 'true'? true: false;
        $result['trash_list']=get_option('clean_trash') === 'true'? true: false;
        $result['draft_list']=get_option('clean_drafts')=== 'true'? true: false;
        $result['spam_list']=get_option('clean_spams')=== 'true'? true: false;
        $result['trash_comments_list']=get_option('clean_trash_comments')=== 'true'? true: false;
        $result['expired_transient_list']=get_option('clean_expired_transients')=== 'true'? true: false;
        $result['all_transient_list']=get_option('clean_all_transients')=== 'true'? true: false;
        $result['optimize_database_list']=get_option('clean_optimize_database')=== 'true'? true: false;
        $result['schedule_cleanup'] = get_option('ENHANCER_SCHEDULE_STATUS')=== 'on'? true: false;

        $schedule_cleanup_timing = get_option('ENHANCER_SCHEDULE_TIME');
        $result['schedule_cleanup_frequency'] = !empty($schedule_cleanup_timing) ? $schedule_cleanup_timing : 'daily';
        
        echo wp_json_encode($result);
        wp_die();     
	}
    
    public function get_db_options(){
            if(isset($_POST)){
                $revisions = sanitize_text_field($_POST['revisions']);
                $drafts = sanitize_text_field($_POST['auto_draft']);
                $trash_post = sanitize_text_field($_POST['trashed_post']);
                $spams = sanitize_text_field($_POST['spam_comments']);
                $trash_comments = sanitize_text_field($_POST['trashed_comments']);
                $expired_transients = sanitize_text_field($_POST['expired_transients']);
                $all_transients = sanitize_text_field($_POST['all_transients']);
                $optimize_database = sanitize_text_field($_POST['optimize_tables']);
                $schedule_status = sanitize_text_field($_POST['schedule_cleanup']);
                $schedule_timing = sanitize_text_field($_POST['schedule_cleanup_frequency']);

                update_option('clean_revisions',$revisions);
                update_option('clean_trash',$trash_post);
                update_option('clean_drafts',$drafts);
                update_option('clean_spams',$spams);
                update_option('clean_trash_comments',$trash_comments);
                update_option('clean_expired_transients',$expired_transients);
                update_option('clean_all_transients',$all_transients);
                update_option('clean_optimize_database',$optimize_database);
                
            }
            $post_cleanup=$this->post_cleanup();

            $schedule_instance = scheduleDBCleanup::getInstance();
            $schedule_instance->getScheduleStatus($schedule_status, $schedule_timing);
    
            $result['success'] = true;
            echo wp_json_encode($result);
            wp_die();
    }

    public function post_cleanup(){
        global $wpdb;
        $revision=get_option('clean_revisions');
        $drafts = get_option('clean_drafts');
        $trash_post = get_option('clean_trash');
        $spams = get_option('clean_spams');
        $trash_comments = get_option('clean_trash_comments');
        $expired_transients = get_option('clean_expired_transients');
        $all_transients = get_option('clean_all_transients');
        $optimize_database = get_option('clean_optimize_database');
        if($revision=="true"){
            $get_revision_ids= $wpdb->get_results($wpdb->prepare("SELECT ID FROM {$wpdb->prefix}posts WHERE post_type=%s", 'revision'));
            foreach($get_revision_ids as $revision_id){
                $deleted_revision=$revision_id->ID;
                wp_delete_post_revision($deleted_revision);
            }
        }
        if($drafts =="true"){
        $get_draft_ids= $wpdb->get_results($wpdb->prepare( "SELECT ID FROM {$wpdb->prefix}posts WHERE post_status=%s", 'draft' ));
        foreach($get_draft_ids as $draft_id){
            $deleted_draft=$draft_id->ID;
            wp_delete_post($deleted_draft);
        }
    }
    if($trash_post =="true"){
        $get_trash_ids= $wpdb->get_results($wpdb->prepare( "SELECT ID FROM {$wpdb->prefix}posts WHERE post_status=%s", 'trash' ));
        foreach($get_trash_ids as $trash_id){
            $deleted_trash=$trash_id->ID;
            wp_delete_post($deleted_trash);
        }
    }
    if($spams =="true"){
        $get_spam_ids= $wpdb->get_results($wpdb->prepare( "SELECT comment_ID FROM {$wpdb->prefix}comments WHERE comment_approved= %s",'spam' ));
        foreach($get_spam_ids as $spam_id){
            $deleted_spam=$spam_id->comment_ID;
            wp_delete_comment($deleted_spam);
        }
    }
    if($trash_comments=="true"){
        $get_trash_comment_ids= $wpdb->get_results($wpdb->prepare( "SELECT comment_ID FROM {$wpdb->prefix}comments WHERE comment_approved= %s",'trash' ));
        foreach($get_trash_comment_ids as $trash_comment_id){
            $deleted_trash_comments=$trash_comment_id->comment_ID;
            wp_delete_comment($deleted_trash_comments);
        }
    }
    if($expired_transients=="true"){
        $time  = isset( $server_array['REQUEST_TIME'] ) ? (int) $server_array['REQUEST_TIME'] : time();
        $expired_transient_name = $wpdb->get_results($wpdb->prepare( $wpdb->prepare( "SELECT option_name FROM $wpdb->options WHERE option_name LIKE %s AND option_value < %d", $wpdb->esc_like( '_transient_timeout' ) . '%', $time )) );
        foreach($expired_transient_name as $expired_transient_name){
            $deleted_expired_transient=$expired_transient_name->option_name;
            delete_expired_transients();
        }
    }
    if($all_transients =="true"){
       $all_transient_name = $wpdb->get_results( $wpdb->prepare( "SELECT option_name FROM $wpdb->options WHERE option_name LIKE %s OR option_name LIKE %s", $wpdb->esc_like( '_transient_' ) . '%', $wpdb->esc_like( '_site_transient_' ) . '%' ) );
       foreach($all_transient_name as $transient_name){
        $deleted_transient_name=$transient_name->option_name;
       delete_transient( str_replace( '_transient_', '',$deleted_transient_name ) );
       }
    }
    if($optimize_database=="true"){
       $get_optimize_table_name=$wpdb->get_results("SELECT table_name, data_free FROM information_schema.tables WHERE table_schema = '" . DB_NAME . "' and Engine <> 'InnoDB' and data_free > 0" );
       foreach($get_optimize_table_name as $optimize_table_name){
        $deleted_tables=$wpdb->query( "OPTIMIZE TABLE $optimize_table_name->table_name" );
       }
    }
       
    }
	

}
$new_obj = new OptimizeDB();