<?php
/**
 * All-in-one Performance Accelerator plugin file.
 *
 * Copyright (C) 2010-2020, Smackcoders Inc - info@smackcoders.com
 */
namespace Smackcoders\AIOACC;

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

class scheduleDBCleanup
{
    protected static $instance = null, $plugin;

    public function __construct() {

    }

    public static function getInstance() {   
        if ( null == self::$instance ) {
            self::$instance = new self;
            self::$plugin = Plugin::getInstance();
        }
        return self::$instance;
    }

    public static function start_scheduler($timing){
		if (!wp_next_scheduled('profile_enhancer_schedule_hook')) {
            wp_schedule_event(time(), $timing , 'profile_enhancer_schedule_hook');
        }
	}

    public function getScheduleStatus($schedule_status, $schedule_timing){
        if($schedule_status == 'true' && isset($schedule_timing)){
			update_option('ENHANCER_SCHEDULE_STATUS', 'on');
            update_option('ENHANCER_SCHEDULE_TIME', $schedule_timing);	
            
            $schedule_db_optimize_timing = array(
                'daily' => 'smack_enhancer_daily',
                'weekly' => 'smack_enhancer_weekly',
                'monthly' => 'smack_enhancer_monthly'
            );
			self::$instance->start_scheduler($schedule_db_optimize_timing[$schedule_timing]);
        }
        elseif($schedule_status == 'false'){
			update_option('ENHANCER_SCHEDULE_STATUS', 'off');
			delete_option('ENHANCER_SCHEDULE_TIME');
			$timestamp = wp_next_scheduled( 'profile_enhancer_schedule_hook' );
    		wp_unschedule_event( $timestamp, 'profile_enhancer_schedule_hook' );
		}	
	}
    
    public function schedule_cleanup(){
        global $wpdb;
        $get_revision_ids= $wpdb->get_results($wpdb->prepare( "SELECT ID FROM {$wpdb->prefix}posts WHERE post_type= %s", 'revision'));
        foreach($get_revision_ids as $revision_id){
            $deleted_revision=$revision_id->ID;
            wp_delete_post_revision($deleted_revision);
        }
        
        $get_draft_ids= $wpdb->get_results($wpdb->prepare( "SELECT ID FROM {$wpdb->prefix}posts WHERE post_status= %s" ,'draft' ));
        foreach($get_draft_ids as $draft_id){
            $deleted_draft=$draft_id->ID;
            wp_delete_post($deleted_draft);
        }
  
    
        $get_trash_ids= $wpdb->get_results($wpdb->prepare( "SELECT ID FROM {$wpdb->prefix}posts WHERE post_status=%s", 'trash'));
        foreach($get_trash_ids as $trash_id){
            $deleted_trash=$trash_id->ID;
            wp_delete_post($deleted_trash);
        }
    
        $get_spam_ids= $wpdb->get_results($wpdb->prepare( "SELECT comment_ID FROM {$wpdb->prefix}comments WHERE comment_approved=%s", 'spam' ));
        foreach($get_spam_ids as $spam_id){
            $deleted_spam=$spam_id->comment_ID;
            wp_delete_comment($deleted_spam);
        }
    
        $get_trash_comment_ids= $wpdb->get_results($wpdb->prepare( "SELECT comment_ID FROM {$wpdb->prefix}comments WHERE comment_approved= %s",'trash' ));
        foreach($get_trash_comment_ids as $trash_comment_id){
            $deleted_trash_comments=$trash_comment_id->comment_ID;
            wp_delete_comment($deleted_trash_comments);
        }
        $server_array = filter_input_array(INPUT_SERVER, FILTER_SANITIZE_STRING);
        $time  = isset( $server_array['REQUEST_TIME'] ) ? (int) $server_array['REQUEST_TIME'] : time();
        $expired_transient_name = $wpdb->get_results( $wpdb->prepare( "SELECT option_name FROM $wpdb->options WHERE option_name LIKE %s AND option_value < %d", $wpdb->esc_like( '_transient_timeout' ) . '%', $time ) );
        foreach($expired_transient_name as $expired_transient_name){
            $deleted_expired_transient=$expired_transient_name->option_name;
            delete_expired_transients();
        }

        $all_transient_name = $wpdb->get_results( $wpdb->prepare( "SELECT option_name FROM $wpdb->options WHERE option_name LIKE %s OR option_name LIKE %s", $wpdb->esc_like( '_transient_' ) . '%', $wpdb->esc_like( '_site_transient_' ) . '%' ) );
        foreach($all_transient_name as $transient_name){
            $deleted_transient_name=$transient_name->option_name;
            delete_transient( str_replace( '_transient_', '',$deleted_transient_name ) );
        }
    
        $get_optimize_table_name=$wpdb->get_results( "SELECT table_name, data_free FROM information_schema.tables WHERE table_schema = '" . DB_NAME . "' and Engine <> 'InnoDB' and data_free > 0" );
        foreach($get_optimize_table_name as $optimize_table_name){
            $deleted_tables=$wpdb->query( "OPTIMIZE TABLE $optimize_table_name->table_name" );
        }
    }
}