<?php
/**
 * All-in-one Performance Accelerator plugin file.
 *
 * Copyright (C) 2010-2020, Smackcoders Inc - info@smackcoders.com
 */

namespace Smackcoders\AIOACC;

use DOMDocument;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

require_once(__DIR__.'/../classes/bufferClass.php');

class LazyImages
{
    protected static $instance = null,$plugin;
    protected $buffer_instance;
	
	public function __construct()
	{
        $this->buffer_instance = new bufferClass([]);
    }
    
    public static function getInstance() {
		if ( null == self::$instance ) {
			self::$instance = new self;
			self::$plugin = Plugin::getInstance();
			self::$instance->doHooks();
		}
        return self::$instance;
	}
	
	public function doHooks(){
		add_action('wp_ajax_save_media_options', array($this,'media_options'));
		add_action('wp_ajax_get_media_options', array($this,'send_media_options'));
	}

	public function media_options(){
		if(isset($_POST)){
			$enable_lazyload = sanitize_text_field($_POST['enable_lazy_load']);
			$disable_emoji = sanitize_text_field($_POST['disable_emoji']);
			$disable_embeds = sanitize_text_field($_POST['disable_embeds']);
			$supportWebpImages = sanitize_text_field($_POST['supportWebpImages']);
			$enable_lazyload_iframe = sanitize_text_field($_POST['enable_lazyload_iframe']);
			update_option('smack_lazyload_enable',$enable_lazyload);
			update_option('smack_emoji_disable',$disable_emoji);
			update_option('smack_embeds_disable',$disable_embeds);
			update_option('smack_webp_image_enable',$supportWebpImages);
			update_option('smack_lazyload_iframe_videos', $enable_lazyload_iframe);
		}
		$result['success'] = true;
		echo wp_json_encode($result);
		wp_die();
	}

	public function send_media_options(){
		$enable_lazyload=get_option('smack_lazyload_enable');
        $disable_emoji=get_option('smack_emoji_disable');
		$disable_embeds=get_option('smack_embeds_disable');
		$supportWebpImages=get_option('smack_webp_image_enable');
		$enable_lazyload_iframes = get_option('smack_lazyload_iframe_videos');
        $result['enable_lazyload']=$enable_lazyload=== 'true'? true: false;
		$result['disable_emoji']=$disable_emoji=== 'true'? true: false;
		$result['disable_embeds']=$disable_embeds=== 'true'? true: false;
		$result['supportWebpImages']=$supportWebpImages=== 'true'? true: false;
		$result['enable_lazyload_iframe'] = $enable_lazyload_iframes === 'true'? true: false;
        $result['success'] = true;
        echo wp_json_encode($result);
        wp_die();
	}

	public function EnableLazyLoad(){
		wp_register_script('iframe-lazyload-script', plugins_url('../assets/js/Iframe.js', __FILE__), array('jquery'),'',true);
        wp_enqueue_script('iframe-lazyload-script');
    }
    
    public function maybe_init_process() {
		ob_start( [ $this, 'maybe_process_buffer' ] );
    }
    
    public function maybe_process_buffer( $buffer ) {
		if ( ! $this->buffer_instance->is_html( $buffer ) ) {
			return $buffer;
		}
		
		if(get_option('smack_lazyload_enable') == 'true'){
			$buffer = $this->smack_lazy_img($buffer);
		}
		if(get_option('smack_lazyload_iframe_videos') == 'true'){
			$buffer = $this->smack_lazy_ifra($buffer);
		}

		return $buffer;
    }
    
	public function smack_lazy_img( $buffer ) {
		$clean_buffer = preg_replace( '/<!--(.*)-->/Uis', '', $buffer );
		preg_match_all('#<img(?<atts>\s.+)\s?/?>#iUs', $clean_buffer, $images);
	
		if ( ! isset( $images[0] ) ) {
			return $buffer;
		}
	
		foreach ( $images[0] as $i => $img ) {
			if(is_array($img)){
				if (!in_array("data-src", $img)){
					$lazy = ["src", "<img"];
					$_img   = ["data-src", '<img loading="lazy"'];
					$lazy_img = str_replace( $lazy, $_img, $img );
		
					$lazy = ["<img", "/>"];
					$_img   = ["<noscript><img", '></noscript>'];
					$lazy_img .= str_replace( $lazy, $_img, $img );
					$buffer  = str_replace( $img, $lazy_img, $buffer );
				}
			}
			
		} 
		return $buffer;
	}

    public function smack_lazy_ifra( $buffer ) {
		$clean_buffer = preg_replace( '/<!--(.*)-->/Uis', '', $buffer );
		preg_match_all( '@<iframe(?<atts>\s.+)>.*</iframe>@iUs', $clean_buffer , $iframes);

		if ( ! isset( $iframes[0] ) ) {
            return $buffer;
		}
		
		foreach ( $iframes[0] as $i => $ifra ) {
            $lazy = ["src", "<iframe"];
            $_ifra   = ["data-src", '<iframe loading="lazy"'];
            $lazy_ifra = str_replace( $lazy, $_ifra, $ifra );
			
			$buffer  = str_replace( $ifra, $lazy_ifra, $buffer );
		} 
        return $buffer;
	}
}
